from selectolax.parser import HTMLParser
import sys
import json
import asyncio
from suap_scraper.config import *

def tratar_notas_detalhar(html_detalhar):
  notas = {}
  tree = HTMLParser(html_detalhar)
  trs_nodes = tree.css('tbody > tr')
  for tr_node in trs_nodes:
    tr_tds = tr_node.css('td')
    nota = tr_tds[4].text()
    sigla_av = tr_tds[0].text()
    notas[sigla_av] = None if nota == '-' else int(nota)

  return notas

async def tratar_etapas_tds(materia_etapas_tds, session):
  etapas = {}
  tasks = []
  etapas_com_link = []
  etapas_td = zip(LABELS_NOTAS_TABLE, materia_etapas_tds)
  for etapa, td in etapas_td:
    link = td.css_first('a')
    texto_td = td.text().strip()
    notas = texto_td
    if link is not None:
      etapas_com_link.append(etapa)
      tasks.append(asyncio.create_task(session.get(LINK_SUAP + link.attributes['href'])))
    etapas[etapa] = notas if notas != '-' else None

  responses = await asyncio.gather(*tasks)
  for i in range(len(etapas_com_link)):
    html_detalhar = responses[i].text
    notas = tratar_notas_detalhar(html_detalhar)
    etapas[etapas_com_link[i]] = notas
  return etapas


def tem_colspan(tag):
  return ('colspan' in list(tag.attributes.keys()))

def criar_indices_colspan(indices_notas, indices_colspans):
  indices_notas_materia = indices_notas[::]
  for i in indices_colspans:
    for j in range(len(indices_notas)):
      if indices_notas_materia[j] > i:
        indices_notas_materia[j] -= 1
  return indices_notas_materia


def materias_etapas_tds(boletim_html):

  tree = HTMLParser(boletim_html)
  materias_tr = tree.css('.borda > tbody > tr')
  materias = {}

  for i in range(len(materias_tr)):

    materia_tr = materias_tr[i]
    materia_tds = materia_tr.css('td')

    disciplina_node = materia_tds[1]
    disciplina = (disciplina_node
    .text()
    .replace('\n', '')
    .strip()
    .replace('  ', ''))

    indices_colspan = [
      i for i in range(len(materia_tds)) if tem_colspan(materia_tds[i])
    ]
    indices_notas_materia = criar_indices_colspan(
      INDICES_NOTAS_PADRAO,
      indices_colspan
    )
    materia_etapas_tds = [materia_tds[i] for i in indices_notas_materia]

    materias[disciplina] = materia_etapas_tds
  return materias

async def parsear_boletim(boletim_html, session):
  materias = materias_etapas_tds(boletim_html)
  tasks = []
  boletim = {}
  disciplinas = materias.keys()

  for disciplina, materia_etapas_tds in materias.items():
    tasks.append(
      asyncio.create_task(
        tratar_etapas_tds(
      materia_etapas_tds,
      session
      )
    ))
  responses = await asyncio.gather(*tasks)
  for i in range(len(disciplinas)):
    boletim[list(disciplinas)[i]] = responses[i]
  return boletim



UA_PADRAO = {'User-Agent': 'Oi, eu sou um bot que raspa o SUAP do IFPB. Disponivel em: https://github.com/Jao42/suap-scraper'}

HEADER_LOGIN = {
  "accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
  "accept-language": "pt-BR,pt;q=0.9,en-US;q=0.8,en;q=0.7",
  "cache-control": "max-age=0",
  "content-type": "application/x-www-form-urlencoded",
  "sec-fetch-dest": "document",
  "sec-fetch-mode": "navigate",
  "sec-fetch-site": "same-origin",
  "sec-fetch-user": "?1",
  "sec-gpc": "1",
  "referer": "https://suap.ifpb.edu.br/accounts/login/?next=/",
  "upgrade-insecure-requests": "1"
  }


REQ_JSON_LOGIN = {
  "referrer": "https://suap.ifpb.edu.br",
  "referrerPolicy": "same-origin",
  "method": "POST",
  "mode": "cors",
  "credentials": "include"
  }


HEADER_BOLETIM = {
  "accept": "*/*",
  "accept-language": "pt-BR,pt;q=0.9,en-US;q=0.8,en;q=0.7",
  "sec-fetch-dest": "empty",
  "sec-fetch-mode": "cors",
  "sec-fetch-site": "same-origin",
  "sec-gpc": "1",
  "x-requested-with": "XMLHttpRequest"
}

REQ_JSON_BOLETIM = {
  "referrerPolicy": "same-origin",
  "method": "GET",
  "mode": "cors",
  "credentials": "include",
  "mode": "cors"
  }

MSGS_ERRO = {"or favor, entre com um usuário  e senha corretos. Note que ambos os campos diferenciam maiúsculas e minúsculas.": "Matricula ou senha estao incorretos",
             "Por favor, corrija o erro abaixo.":"Captcha impediu o login(credenciais validas)"
             }

INDICES_NOTAS_PADRAO = [7, 9, 11, 13, 15, 16, 18]
LABELS_NOTAS_TABLE = ['E1', 'E2', 'E3', 'E4', 'MD', 'NAF', 'CONCEITO']
LINK_SUAP = 'https://suap.ifpb.edu.br'



import httpx
import asyncio
from selectolax.parser import HTMLParser
import sys
from suap_scraper.utils import parsear_boletim
import json
import os
from suap_scraper.config import *


class LoginError(Exception):
  """
  Gerar mensagens de erro ao n conseguir logar
  """


class SUAP:
  def __init__(self, user_agent=UA_PADRAO):
    self.user_agent = user_agent
    self.session = httpx.AsyncClient()
    self.session.headers.update(user_agent)
    self.session_id = None
    self.matricula = None
    self.csrf = None
    self.senha = None

  async def loginCredenciais(self, matricula, senha, user_agent=UA_PADRAO):
    self.matricula = matricula
    self.senha = senha
    self.session_id, self.csrf = await self.__loginSUAP()

  async def loginSessionId(self, session_id, user_agent=UA_PADRAO):
    self.session_id = session_id
    self.session.cookies['sessionid'] = self.session_id

  async def __getInitialPage(self):
    res = await self.session.get(LINK_SUAP)
    return res

  def __getCookiesInitialPage(self, initial_page):
    tree = HTMLParser(initial_page.text)
    middleware_csrf = (
      tree.css('input[name="csrfmiddlewaretoken"]')
      .attributes['value']
    )

    csrf = initial_page.cookies['csrftoken']
    session_id = initial_page.cookies['sessionid']

    return [ middleware_csrf, csrf, session_id ]


  async def __loginSUAP(self, campo_captcha=False):
    initial_page = await self.__getInitialPage()
    middleware_csrf, csrf, session_id = self.__getCookiesInitialPage(
      initial_page
    )

    body = ("csrfmiddlewaretoken=" + middleware_csrf +
    "&username=" + str(self.matricula) +
    "&password=" + self.senha +
    "&this_is_the_login_form=1&next="
    )

    if campo_captcha:
      body += "&g-recaptcha-response="

    req = session.build_request("POST",
                           LINK_SUAP + "/accounts/login/?next=/",
                           headers=HEADER_LOGIN,
                           data=body,
                           json=REQ_JSON_LOGIN,
                           cookies={
                            'csrftoken': csrf,
                            'sessionid': session_id
                          }
                        )

    res = await self.session.send(req)

    try:
      session_id = res.cookies['sessionid']
    except KeyError:
      tree = HTMLParser(res.text)
      erro = tree.css(".errornote")
      if not erro:
        raise LoginError("Não foi possivel logar no SUAP")
      msg_erro = erro.get_text().strip()
      try:
        raise LoginError(MSGS_ERRO[msg_erro])
      except KeyError:
        raise LoginError(msg_erro)
    try:
      csrf = res.cookies['csrftoken']
    except:
      pass

    return [ session_id, csrf ]

  async def __getBoletimPage(self):
    if self.matricula is None:
      logged = await self.__getInitialPage()
      self.matricula = logged.headers.get('user')

    res = await self.session.get(LINK_SUAP + "/edu/aluno/" + str(self.matricula) +"/?tab=boletim")
    boletim_html = res.text
    return boletim_html

  async def __createBoletimJSON(self, html_content):
    dic_materias = await parsear_boletim(html_content, self.session)
    boletim_json = json.dumps(
      dic_materias, sort_keys=True, indent=2, ensure_ascii=False
    )

    return boletim_json

  async def getBoletim(self):
    html_boletim = await self.__getBoletimPage()
    boletim = await self.__createBoletimJSON(html_boletim)
    return boletim

